import java.util.ArrayList;
import java.util.Stack;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

public class AutoComplete {
    public static void main(String args[]) throws IOException {
        Trie head = new Trie();
	
		long c0 = System.currentTimeMillis();
		BufferedReader reader = null;
		try {
           reader = new BufferedReader(new FileReader("words.txt"));
           String line;
           while ((line = reader.readLine()) != null) {
               insertInTrie(line, head);
           }
    	} finally {
           try {
               if (reader != null) reader.close();
           } catch (IOException e) {
           }
    	}
    	long c1 = System.currentTimeMillis();
		System.out.println("Indexing Time in milliseconds = "+(c1-c0));

		c0 = System.currentTimeMillis();
		try {
			reader = new BufferedReader(new FileReader("prefixQueries.txt"));
    		String prefixQuery;
			while ((prefixQuery = reader.readLine()) != null) {
    			ArrayList<String> v = autoCompleteSuggestions(prefixQuery, head);
				System.out.println(v.size());
				/*for (String aV : v)
            		System.out.println(aV);*/
			}
		} finally {
			try {
				if (reader != null) reader.close();
			} catch (IOException e) {
			}
		}    	
		c1 = System.currentTimeMillis();
		System.out.println("Search Time in milliseconds = "+(c1-c0));
    }

    public static void insertInTrie(String token, Trie currentNode) {

        //extracting characters of the token in the char buffer
        char buf[] = new char[token.length()];
        token.getChars(0, token.length(), buf, 0);

        for (int i = 0; i < token.length(); i++) {

            char ch = buf[i];
            int k = (int) ch;

            //if the token till this character is already inserted in Trie
            if (currentNode.next[k] != null) {
                currentNode = currentNode.next[k];
                /**
                 * the last node of each token has the end boolean true representing end of word
                 * the last node also contains the complete token
                 */
                if (i == token.length() - 1) {
                    currentNode.token = token;
                }
            } else {
                Trie newNode = new Trie();
                if (i == token.length() - 1) {
                    newNode.token = token;
                }
                currentNode.next[k] = newNode;
                currentNode = newNode;
            }
        }

    }

    /**
     * The autoCompleteSuggestions() returns Vector of Strings containing suggestions for AutoComplete
     * The prefixQuery is first searched in trie whether it is prefix of some token or not
     * If prefix is found to exist, a dfs search is implemented to get complete suggestions
     */
    public static ArrayList<String> autoCompleteSuggestions(String prefixQuery, Trie currentNode) {

        //extracting characters of the token in the char buffer
        char buf[] = new char[prefixQuery.length()];
        prefixQuery.getChars(0, prefixQuery.length(), buf, 0);

        ArrayList<String> suggestions = new ArrayList<String>();

        /**
         * finding if the prefixQuery exists in the trie as prefix of some token
         * if the prefixQuery is present in the trie, then i equals length of prefixQuery
         * else i < length of prefixQuery
         */
        int i;
        for (i = 0; i < prefixQuery.length(); i++) {

            char ch = buf[i];
            int k = (int) ch;

            if (currentNode.next[k] == null)
                break;
            currentNode = currentNode.next[k];
        }
        /** dfs to get the complete suggestions */
        if (i == prefixQuery.length() && currentNode != null) {
			
            Stack st = new Stack();
            st.push(currentNode);

            while (!st.empty()) {
                Trie top = (Trie) st.peek();
				if (top.token != null)
					suggestions.add(top.token);
                boolean flag = true;
				st.pop();
                for (int j = 0; j < 256; j++) {
                    if (top.next[j] != null) {
                        st.push(top.next[j]);
                    }
                }
            }
        }
        return suggestions;
    }
}


class Trie {
    public Trie next[] = new Trie[256];
    public String token;
}


